package sun.swing;

import java.awt.Image;
import java.util.ListIterator;
import java.awt.GraphicsConfiguration;
import java.lang.ref.SoftReference;
import java.util.LinkedList;

public class ImageCache
{
    private int maxCount;
    private final LinkedList<SoftReference<ImageCache.Entry>> entries;
    
    public ImageCache(int maxCount) {
        this.maxCount = maxCount;
        this.entries = new LinkedList();
    }
    
    void setMaxCount(int maxCount) {
        this.maxCount = maxCount;
    }
    
    public void flush() {
        this.entries.clear();
    }
    
    private ImageCache.Entry getEntry(Object o, GraphicsConfiguration graphicsConfiguration, int n, int n2, Object[] array) {
        ListIterator<Object> listIterator = this.entries.listIterator();
        while (listIterator.hasNext()) {
            SoftReference<Object> e = (SoftReference)listIterator.next();
            ImageCache.Entry entry = (ImageCache.Entry)e.get();
            if (entry == null) {
                listIterator.remove();
            }
            else {
                if (entry.equals(graphicsConfiguration, n, n2, array)) {
                    listIterator.remove();
                    this.entries.addFirst(e);
                    return entry;
                }
                continue;
            }
        }
        ImageCache.Entry referent = new ImageCache.Entry(graphicsConfiguration, n, n2, array);
        if (this.entries.size() >= this.maxCount) {
            this.entries.removeLast();
        }
        this.entries.addFirst(new SoftReference(referent));
        return referent;
    }
    
    public Image getImage(Object o, GraphicsConfiguration graphicsConfiguration, int n, int n2, Object[] array) {
        return this.getEntry(o, graphicsConfiguration, n, n2, array).getImage();
    }
    
    public void setImage(Object o, GraphicsConfiguration graphicsConfiguration, int n, int n2, Object[] array, Image image) {
        this.getEntry(o, graphicsConfiguration, n, n2, array).setImage(image);
    }
    
    private static class Entry
    {
        private final GraphicsConfiguration config;
        private final int w;
        private final int h;
        private final Object[] args;
        private Image image;
        
        Entry(GraphicsConfiguration config, int w, int h, Object[] args) {
            this.config = config;
            this.args = args;
            this.w = w;
            this.h = h;
        }
        
        public void setImage(Image image) {
            this.image = image;
        }
        
        public Image getImage() {
            return this.image;
        }
        
        @Override
        public String toString() {
            String s = super.toString() + "[ graphicsConfig=" + this.config + ", image=" + this.image + ", w=" + this.w + ", h=" + this.h;
            if (this.args != null) {
                for (int i = 0; i < this.args.length; ++i) {
                    s = s + ", " + this.args[i];
                }
            }
            return s + "]";
        }
        
        public boolean equals(GraphicsConfiguration obj, int n, int n2, Object[] array) {
            if (this.w == n && this.h == n2 && ((this.config != null && this.config.equals(obj)) || (this.config == null && obj == null))) {
                if (this.args == null && array == null) {
                    return true;
                }
                if (this.args != null && array != null && this.args.length == array.length) {
                    int i = array.length - 1;
                    while (i >= 0) {
                        Object o = this.args[i];
                        Object obj2 = array[i];
                        if ((o == null && obj2 != null) || (o != null && !o.equals(obj2))) {
                            return false;
                        }
                        --i;
                    }
                    return true;
                }
            }
            return false;
        }
    }
}
